# function to simulate a pure-birth phylogenetic tree or trees
# written by Liam J. Revell 2011-2013

pbtree<-function(b=1,n=NULL,t=NULL,scale=NULL,nsim=1,type=c("continuous","discrete"),...){
	if(hasArg(ape)) ape<-list(...)$ape
	else ape<-TRUE
	if(hasArg(quiet)) quiet<-list(...)$quiet
	else quiet<-FALSE
	type<-matchType(type[1],c("continuous","discrete"))
	if(nsim>1){
		trees<-replicate(nsim,pbtree(b,n,t,scale,type=type,ape=ape),simplify=FALSE)
		class(trees)<-"multiPhylo"
		return(trees)
	} else {
		if(!is.null(n)&&!is.null(t)){
			if(!quiet){
				cat("simulating with both taxa-stop (n) and time-stop (t) is\n")
				cat("performed via rejection sampling & may be slow\n\n")
			}
			N<-0; count<--1
			while(N!=n){ 
				tree<-pbtree(b,t=t,ape=FALSE)
				N<-length(tree$tip.label)
				count<-count+1
			}
			if(!quiet) cat(paste("  ",count," trees rejected before finding a tree\n\n",sep=""))
		} else {
			if(!is.null(t)){
				node<-1
				edge<-matrix(c(node,NA,node,NA),2,2,byrow=T)
				edge.length<-c(0,0)
				node<-node+1; tt<-0
				while(tt<t){
					o<-is.na(edge[,2])
					p<-which(o)
					if(type=="discrete"){
						l<-rgeom(n=sum(o),prob=b)+1
						l<-l[which(l==min(l))]
					} else l<-rexp(n=1,sum(o)*b)
					tt<-tt+l[1]
					if(tt>=t) l<-l-tt+t
					else {
						q<-sample(p)[1:length(l)]
						for(i in 1:length(l)){
							# new edge
							edge[q[i],2]<-node
							edge<-rbind(edge,matrix(c(node,NA,node,NA),2,2,byrow=T))
							node<-node+1
						}
					}
					edge.length[p]<-edge.length[p]+l[1]
					edge.length<-c(edge.length,rep(0,2*length(l)))
				}
				o<-is.na(edge[,2])
				n<-sum(o)
				edge<-edge+n
				p<-which(o)
				edge[is.na(edge[,2]),2]<-1:sum(is.na(edge[,2]))
			} else {
				node<-1
				edge<-matrix(c(node,NA,node,NA),2,2,byrow=T)
				edge.length<-c(0,0)
				node<-node+1
				while(nrow(edge)<(2*n-2)){
					o<-is.na(edge[,2])
					p<-which(o)
					if(type=="discrete"){
						l<-rgeom(n=sum(o),prob=b)+1
						l<-l[which(l==min(l))]
					} else l<-rexp(n=1,sum(o)*b)
					q<-sample(p)[1:length(l)]
					for(i in 1:length(l)){
						# new edge
						edge[q[i],2]<-node
						edge<-rbind(edge,matrix(c(node,NA,node,NA),2,2,byrow=T))
						node<-node+1
					}
					edge.length[p]<-edge.length[p]+l[1]
					edge.length<-c(edge.length,rep(0,2*length(l)))
				}
				o<-is.na(edge[,2])
				nn<-sum(o)
				edge<-edge+nn
				p<-which(o)
				l<-if(type=="discrete") min(rgeom(n=sum(o),prob=b)+1) else rexp(n=1,sum(o)*b)			
				edge.length[p]<-edge.length[p]+l		
				edge[is.na(edge[,2]),2]<-1:sum(is.na(edge[,2]))
				if(nn!=n){ 
					cat("Warning:\n  due to multiple speciation events in the final time interval\n")
					cat("  realized n may not equal input n\n\n")
				}
				n<-nn
			}
			tip.label<-paste("t",1:n,sep="")
			tree<-list(edge=edge,edge.length=edge.length,tip.label=tip.label,Nnode=n-1)
			class(tree)<-"phylo"
			if(!is.null(scale)){
				h<-max(nodeHeights(tree))
				tree$edge.length<-scale*tree$edge.length/h
			}
		}
		if(ape) tree<-read.tree(text=write.tree(tree))
		return(tree)
	}
}
