\name{pbtree}
\alias{pbtree}
\title{Simulate pure-birth stochastic tree or trees}
\usage{
	pbtree(b=1, n=NULL, t=NULL, scale=NULL, nsim=1, type=c("continuous","discrete"), ...)
}
\arguments{
	\item{b}{birth rate or speciation rate for \code{type="continuous"}; the probability of speciating per time-step for \code{type="discrete"}.}
	\item{n}{desired number of species (i.e., taxa-stop criterion).}
	\item{t}{total time for simulation (i.e., time-stop criterion).}
	\item{scale}{if set, rescales tree to have total length \code{scale}.}
	\item{nsim}{number of simulated trees to return.}
	\item{type}{string to indicate whether to simulate trees in continuous or discrete time. If the former, then wait times between speciation events are drawn from an exponential distribution; whereas if the latter then wait times comes from a geometric distribution.}
	\item{...}{optional arguments including \code{ape}, a logical value indicating whether to return nodes in a 'ape' compatible ordering (default is \code{TRUE}); and \code{quiet}, a logical value indicating whether or not to suppress certain message (defaults to \code{FALSE}).}
}
\description{
	This function simulates stochastic pure-birth (Yule) trees. Simulation can be performed conditioning on \code{n}, on \code{t}, or on both simultaneously. If the both, then rejection sampling is performed whereby trees are simulated given \code{b} and \code{t} until a tree containing \code{n} taxa is found. There is no giving-up point, so this should be used cautiously. Simulations can also be performed in continuous time (the default) or discrete time; the difference between that wait times in the continous-time simulation come from the exponential distribution; whereas waiting times in discrete-time simulations come from the geometric distribution. In addition, discrete-time simulations allow for the possibility that multiple speciation events can occur at (exactly) the same time, so long as they are on separate branches. Finally, sometimes for stopping criterion \code{n} in discrete-time there will be a number of tips different from \code{n}. This indicates that the last event contained more than one speciation event, and a warning is printed.

	Note that if \code{ape=FALSE}, then the function will run faster, and the tree is theoretically compatible with the ape \code{"phylo"} standard; however some downstream errors with functions such as \code{\link{bind.tree}} have been observed.
}
\value{
	a tree or list of trees.
}
\author{Liam Revell \email{liam.revell@umb.edu}}
\examples{
	# simulate a pure-birth tree with 1000 tips, scaled to a length of 1.0
	tree<-pbtree(n=1000,scale=1)
	# simulate a pure-birth tree conditioning on n & t
	tt<-log(50)-log(2)
	tree<-pbtree(n=50,t=tt)
}
\keyword{phylogenetics}
\keyword{simulation}
